# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from StanfordDependencies import (StanfordDependencies, get_instance,
                                  JavaRuntimeVersionError)
from StanfordDependencies.SubprocessBackend import SubprocessBackend
from StanfordDependencies.JPypeBackend import JPypeBackend

# these trees are produced with SD 3.4.1 since it allows us to test
# on older JREs
tree1 = '(S1 (NP (DT a) (NN cow)))'
tree1_out = '''
Token(index=1, form='a', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='cow', cpos='NN', pos='NN', head=0, deprel='root')
'''.strip()

tree2 = '(S1 (NP (NP (NP (DT A) (NN cat)) (CC and) (NP (DT a) ' \
        '(NN mouse))) (. .)))'
tree2_out_basic = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='cat', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=3, form='and', cpos='CC', pos='CC', head=2, deprel='cc')
Token(index=4, form='a', cpos='DT', pos='DT', head=5, deprel='det')
Token(index=5, form='mouse', cpos='NN', pos='NN', head=2, deprel='conj')
Token(index=6, form='.', cpos='.', pos='.', head=2, deprel='punct')'''.strip()
tree2_out_collapsed = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='cat', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=4, form='a', cpos='DT', pos='DT', head=5, deprel='det')
Token(index=5, form='mouse', cpos='NN', pos='NN', head=2, deprel='conj_and')
Token(index=6, form='.', cpos='.', pos='.', head=2, deprel='punct')'''.strip()
tree2_out_CCprocessed = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='cat', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=4, form='a', cpos='DT', pos='DT', head=5, deprel='det')
Token(index=5, form='mouse', cpos='NN', pos='NN', head=2, deprel='conj_and')
Token(index=6, form='.', cpos='.', pos='.', head=2, deprel='punct')'''.strip()
tree2_out_collapsedTree = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='cat', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=4, form='a', cpos='DT', pos='DT', head=5, deprel='det')
Token(index=5, form='mouse', cpos='NN', pos='NN', head=2, deprel='conj_and')
Token(index=6, form='.', cpos='.', pos='.', head=2, deprel='punct')'''.strip()

tree3 = '(S1 (NP (DT some) (JJ blue) (NN moose)))'
tree3_out = '''
Token(index=1, form='some', cpos='DT', pos='DT', head=3, deprel='det')
Token(index=2, form='blue', cpos='JJ', pos='JJ', head=3, deprel='amod')
Token(index=3, form='moose', cpos='NN', pos='NN', head=0, deprel='root')
'''.strip()

tree4 = '(S1 (NP (NP (DT A) (NN burrito)) (PP (IN with) (NP (NP ' + \
        '(NNS beans)) (CONJP (CC but) (RB not)) (NP (NN chicken)))) (. .)))'
tree4_out_basic = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='burrito', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=3, form='with', cpos='IN', pos='IN', head=2, deprel='prep')
Token(index=4, form='beans', cpos='NNS', pos='NNS', head=3, deprel='pobj')
Token(index=5, form='but', cpos='CC', pos='CC', head=6, deprel='cc')
Token(index=6, form='not', cpos='RB', pos='RB', head=4, deprel='cc')
Token(index=7, form='chicken', cpos='NN', pos='NN', head=4, deprel='conj')
Token(index=8, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree4_out_collapsed = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='burrito', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=4, form='beans', cpos='NNS', pos='NNS', head=2, deprel='prep_with')
Token(index=7, form='chicken', cpos='NN', pos='NN', head=4, deprel='conj_negcc')
Token(index=8, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree4_out_CCprocessed = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='burrito', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=4, form='beans', cpos='NNS', pos='NNS', head=2, deprel='prep_with')
Token(index=7, form='chicken', cpos='NN', pos='NN', head=2, deprel='prep_with')
Token(index=7, form='chicken', cpos='NN', pos='NN', head=4, deprel='conj_negcc')
Token(index=8, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree4_out_collapsedTree = '''
Token(index=1, form='A', cpos='DT', pos='DT', head=2, deprel='det')
Token(index=2, form='burrito', cpos='NN', pos='NN', head=0, deprel='root')
Token(index=4, form='beans', cpos='NNS', pos='NNS', head=2, deprel='prep_with')
Token(index=7, form='chicken', cpos='NN', pos='NN', head=4, deprel='conj_negcc')
Token(index=8, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()

tree5 = '''
(S1 (S (NP (NNP Ed))
     (VP (VBZ cooks)
      (CC and)
      (VBZ sells)
      (NP (NP (NNS burritos))
       (PP (IN with)
	(NP (NNS beans) (CONJP (CC but) (RB not)) (NN rice)))))
     (. .)))
'''.strip()
tree5_out_basic = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=3, form='and', cpos='CC', pos='CC', head=2, deprel='cc')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=6, form='with', cpos='IN', pos='IN', head=5, deprel='prep')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=6, deprel='pobj')
Token(index=8, form='but', cpos='CC', pos='CC', head=9, deprel='cc')
Token(index=9, form='not', cpos='RB', pos='RB', head=7, deprel='cc')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj')
Token(index=11, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree5_out_collapsed = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj_and')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=5, deprel='prep_with')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj_negcc')
Token(index=11, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree5_out_CCprocessed = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=4, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj_and')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=5, deprel='prep_with')
Token(index=10, form='rice', cpos='NN', pos='NN', head=5, deprel='prep_with')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj_negcc')
Token(index=11, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree5_out_collapsedTree = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj_and')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=5, deprel='prep_with')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj_negcc')
Token(index=11, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree5_out_collapsedTree_no_punc = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj_and')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=5, deprel='prep_with')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj_negcc')
'''.strip()
tree5_out_collapsedTree_erased = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=3, form='and', cpos='CC', pos='CC', head=0, deprel='erased')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj_and')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=6, form='with', cpos='IN', pos='IN', head=0, deprel='erased')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=5, deprel='prep_with')
Token(index=8, form='but', cpos='CC', pos='CC', head=0, deprel='erased')
Token(index=9, form='not', cpos='RB', pos='RB', head=0, deprel='erased')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj_negcc')
Token(index=11, form='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()
tree5_out_collapsedTree_erased_no_punct = '''
Token(index=1, form='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=3, form='and', cpos='CC', pos='CC', head=0, deprel='erased')
Token(index=4, form='sells', cpos='VBZ', pos='VBZ', head=2, deprel='conj_and')
Token(index=5, form='burritos', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=6, form='with', cpos='IN', pos='IN', head=0, deprel='erased')
Token(index=7, form='beans', cpos='NNS', pos='NNS', head=5, deprel='prep_with')
Token(index=8, form='but', cpos='CC', pos='CC', head=0, deprel='erased')
Token(index=9, form='not', cpos='RB', pos='RB', head=0, deprel='erased')
Token(index=10, form='rice', cpos='NN', pos='NN', head=7, deprel='conj_negcc')
'''.strip()
tree5_out_basic_lemmas = '''
Token(index=1, form='Ed', lemma='Ed', cpos='NNP', pos='NNP', head=2, deprel='nsubj')
Token(index=2, form='cooks', lemma='cook', cpos='VBZ', pos='VBZ', head=0, deprel='root')
Token(index=3, form='and', lemma='and', cpos='CC', pos='CC', head=2, deprel='cc')
Token(index=4, form='sells', lemma='sell', cpos='VBZ', pos='VBZ', head=2, deprel='conj')
Token(index=5, form='burritos', lemma='burrito', cpos='NNS', pos='NNS', head=2, deprel='dobj')
Token(index=6, form='with', lemma='with', cpos='IN', pos='IN', head=5, deprel='prep')
Token(index=7, form='beans', lemma='bean', cpos='NNS', pos='NNS', head=6, deprel='pobj')
Token(index=8, form='but', lemma='but', cpos='CC', pos='CC', head=9, deprel='cc')
Token(index=9, form='not', lemma='not', cpos='RB', pos='RB', head=7, deprel='cc')
Token(index=10, form='rice', lemma='rice', cpos='NN', pos='NN', head=7, deprel='conj')
Token(index=11, form='.', lemma='.', cpos='.', pos='.', head=2, deprel='punct')
'''.strip()

basic_tests = ((tree1, tree1_out), (tree2, tree2_out_basic), (tree3, tree3_out),
               (tree4, tree4_out_basic), (tree5, tree5_out_basic))
basic_tests = [(tree5, tree5_out_basic)]
repr_tests2 = dict(basic=tree2_out_basic, collapsed=tree2_out_collapsed,
                   CCprocessed=tree2_out_CCprocessed,
                   collapsedTree=tree2_out_collapsedTree)
repr_tests4 = dict(basic=tree4_out_basic, collapsed=tree4_out_collapsed,
                   CCprocessed=tree4_out_CCprocessed,
                   collapsedTree=tree4_out_collapsedTree)
repr_tests5 = dict(basic=tree5_out_basic, collapsed=tree5_out_collapsed,
                   CCprocessed=tree5_out_CCprocessed,
                   collapsedTree=tree5_out_collapsedTree)

def test_subprocess_backend_creation():
    sd = StanfordDependencies.get_instance(backend='subprocess',
                                           version='3.4.1',
                                           download_if_missing=True)
    assert isinstance(sd, SubprocessBackend)

def test_subprocess_backend_creation_shortcut():
    sd = get_instance(backend='subprocess', version='3.4.1',
                      download_if_missing=True)
    assert isinstance(sd, SubprocessBackend)

def test_subprocess_backend_creation_newest_version():
    sd = get_instance(backend='subprocess', version=None,
                      download_if_missing=True)
    assert isinstance(sd, SubprocessBackend)

def test_jpype_backend_creation():
    sd = StanfordDependencies.get_instance(backend='jpype',
                                           version='3.4.1',
                                           download_if_missing=True)
    assert isinstance(sd, JPypeBackend), \
           "Fell back to another backend due to a JPype error"

def stringify_sentence(tokens):
    from StanfordDependencies import CoNLL
    # this doesn't work in general, but for current tests we want all
    # the string fields to be strings, not unicode
    new_tokens = []
    for token in tokens:
        new_fields = []
        for field in CoNLL.FIELD_NAMES:
            value = getattr(token, field)
            if isinstance(value, basestring):
                value = str(value)
            new_fields.append(value)
        new_tokens.append(CoNLL.Token(*new_fields))
    return '\n'.join(repr(token) for token in new_tokens)

class DefaultBackendTest(unittest.TestCase):
    backend = None
    version = '3.4.1'
    def setUp(self):
        kwargs = dict(version=self.version, download_if_missing=True)
        if self.backend is not None:
            kwargs['backend'] = self.backend
        self.sd = StanfordDependencies.get_instance(**kwargs)
    def test_basic_single(self):
        for tree, expected in basic_tests:
            self.assertConverts(tree, expected)
    def test_basic_multiple(self):
        trees, expected_outputs = zip(*basic_tests)
        sentences = self.sd.convert_trees(trees)
        for tokens, expected in zip(sentences, expected_outputs):
            self.assertTokensMatch(tokens, expected)
    def test_reprs(self):
        for representation, expected in sorted(repr_tests2.items()):
            self.assertConverts(tree2, expected, representation=representation)
        for representation, expected in sorted(repr_tests4.items()):
            self.assertConverts(tree4, expected, representation=representation)
        for representation, expected in sorted(repr_tests5.items()):
            self.assertConverts(tree5, expected, representation=representation)
    def test_punct_and_erased(self):
        self.assertConverts(tree5, tree5_out_collapsedTree_no_punc,
                            representation='collapsedTree',
                            include_punct=False, include_erased=False)
        self.assertConverts(tree5, tree5_out_collapsedTree_erased_no_punct,
                            representation='collapsedTree',
                            include_punct=False, include_erased=True)
        self.assertConverts(tree5, tree5_out_collapsedTree,
                            representation='collapsedTree',
                            include_punct=True, include_erased=False)
        self.assertConverts(tree5, tree5_out_collapsedTree_erased,
                            representation='collapsedTree',
                            include_punct=True, include_erased=True)
    def test_bogus_representation(self):
        self.assertRaises(ValueError, self.sd.convert_tree, tree1,
                          representation='bogus')
    def test_bogus_backend_creation(self):
        self.assertRaises(ValueError, StanfordDependencies.get_instance,
                          backend='bogus')
    def test_bogus_version(self):
        self.assertRaises(ValueError, StanfordDependencies.get_instance,
                          version='bogus')
        self.assertRaises(ValueError, StanfordDependencies.get_instance,
                          version='0')
        self.assertRaises(TypeError, StanfordDependencies.get_instance,
                          version=0)
    def test_insufficient_jar_info(self):
        self.assertRaises(ValueError, StanfordDependencies.get_instance,
                          backend=self.backend, jar_filename=None,
                          download_if_missing=False, version=None)
    def test_get_jar_url(self):
        assert self.sd.get_jar_url(version='3.5.0') == \
            "http://search.maven.org/remotecontent?filepath=edu/" \
            "stanford/nlp/stanford-corenlp/3.5.0/stanford-corenlp-3.5.0.jar"
        # unclear what it should be since it can change but it should
        # at least be a true value
        assert self.sd.get_jar_url()
    def test_get_jar_url_bad_version(self):
        self.assertRaises(TypeError, self.sd.get_jar_url, 0)
        self.assertRaises(TypeError, self.sd.get_jar_url, -1)
        self.assertRaises(TypeError, self.sd.get_jar_url, 10)
        self.assertRaises(TypeError, self.sd.get_jar_url, (3, 5, 0))

    def assertConverts(self, tree, expected, **conversion_options):
        print 'tree:'
        print tree
        print 'conversion_options:'
        print conversion_options
        tokens = self.sd.convert_tree(tree, **conversion_options)
        self.assertTokensMatch(tokens, expected)
    def assertTokensMatch(self, tokens, expected_stringification):
        stringified = stringify_sentence(tokens)
        print 'actual stringified:'
        print stringified
        print 'expected stringified:'
        print expected_stringification
        print 'matches:', stringified == expected_stringification
        assert stringified == expected_stringification
        print

class SubprocessBackendTest(DefaultBackendTest):
    backend = 'subprocess'
    version = '3.4.1'

    def test_raise_on_bad_exitcode(self):
        self.assertRaises(ValueError,
                          self.sd._raise_on_bad_exitcode, 100, '')
        self.assertRaises(ValueError,
                          self.sd._raise_on_bad_exitcode, 1, '')
        self.assertRaises(ValueError,
                          self.sd._raise_on_bad_exitcode, -1, '')
        self.assertRaises(JavaRuntimeVersionError,
                          self.sd._raise_on_bad_exitcode, 1,
                          'Unsupported major.minor version')
        self.assertRaises(JavaRuntimeVersionError,
                          self.sd._raise_on_bad_exitcode, -7,
                          'Unsupported major.minor version',
                          debug=True)
        self.sd._raise_on_bad_exitcode(0, '') # shouldn't raise anything

    def test_convert_debug(self):
        self.assertConverts(tree1, tree1_out, debug=True)

class JPypeBackendTest(DefaultBackendTest):
    backend = 'jpype'
    version = '3.4.1'

    def test_add_lemmas(self):
        self.assertConverts(tree5, tree5_out_basic_lemmas, add_lemmas=True)
    def test_report_version_error(self):
        self.assertRaises(JavaRuntimeVersionError,
                          self.sd._report_version_error, '1.6')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
from StanfordDependencies.CoNLL import Corpus, Sentence
from test_stanforddependencies import (tree4, tree4_out_CCprocessed,
                                       tree5, tree5_out_CCprocessed,
                                       tree5_out_collapsedTree_no_punc,
                                       stringify_sentence)

older_than_py27 = sys.version_info[:2] < (2, 7)

# example from http://ilk.uvt.nl/conll/example.html
conll_example = '''
1	Cathy	Cathy	N	N	eigen|ev|neut	2	su	_	_
2	zag	zie	V	V	trans|ovt|1of2of3|ev	0	ROOT	_	_
3	hen	hen	Pron	Pron	per|3|mv|datofacc	2	obj1	_	_
4	wild	wild	Adj	Adj	attr|stell|onverv	5	mod	_	_
5	zwaaien	zwaai	N	N	soort|mv|neut	2	vc	_	_
6	.	.	Punc	Punc	punt	5	punct	_	_
'''

def test_conll_readwrite_sentence():
    sentence = Sentence.from_conll(conll_example.splitlines())
    assert sentence.as_conll() == conll_example.strip()

def test_conll_readwrite_corpus():
    corpus = Corpus.from_conll(conll_example.splitlines())
    assert corpus.as_conll() == conll_example.strip()

def test_conll_as_asciitree():
    asciitree_out = '''
 zag [ROOT]
  +-- Cathy [su]
  +-- hen [obj1]
  +-- zwaaien [vc]
     +-- wild [mod]
     +-- . [punct]'''

    sentence = Sentence.from_conll(conll_example.splitlines())
    assert sentence.as_asciitree().strip() == asciitree_out.strip()

def test_conll_as_asciitree_custom():
    asciitree_out = '''
ROOT:zag:V
  +--su:Cathy:N
  +--obj1:hen:Pron
  +--vc:zwaaien:N
     +--mod:wild:Adj
     +--punct:.:Punc'''

    def str_func(token):
        return '%s:%s:%s' % (token.deprel, token.form, token.cpos)
    sentence = Sentence.from_conll(conll_example.splitlines())
    assert sentence.as_asciitree(str_func=str_func).strip() == \
           asciitree_out.strip()

def test_conll_as_asciitree_nontree():
    sample_deps = '''
det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)
    '''.strip().splitlines()
    sentence = Sentence.from_stanford_dependencies(sample_deps, tree4)
    assert len(sentence) == 6
    assert sentence.as_asciitree().strip() == '''
 burrito [root]
  +-- A [det]
  +-- beans [prep_with]
  |  +-- chicken [conj_negcc]
  +-- chicken [prep_with]
  +-- . [punct]
'''.strip()

def test_conll_as_asciitree_nontree_erased():
    sample_deps = '''
det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)
    '''.strip().splitlines()
    sentence = Sentence.from_stanford_dependencies(sample_deps, tree4,
                                                   include_erased=True)
    assert len(sentence) == 9
    assert sentence.as_asciitree().strip() == '''
 ROOT [ROOT-DEPREL]
  +-- burrito [root]
  |  +-- A [det]
  |  +-- beans [prep_with]
  |  |  +-- chicken [conj_negcc]
  |  +-- chicken [prep_with]
  |  +-- . [punct]
  +-- with [erased]
  +-- but [erased]
  +-- not [erased]
'''.strip()

def test_conll_as_dotgraph_nontree():
    if older_than_py27: # this feature disabled in older Pythons
        return
    sample_deps = '''
det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)
    '''.strip().splitlines()
    sentence = Sentence.from_stanford_dependencies(sample_deps, tree4)
    assert len(sentence) == 6
    assert sentence.as_dotgraph().source == '''
digraph {
	0 [label=root]
	1 [label=A]
		2 -> 1 [label=det]
	2 [label=burrito]
		0 -> 2 [label=root]
	4 [label=beans]
		2 -> 4 [label=prep_with]
	7 [label=chicken]
		2 -> 7 [label=prep_with]
		4 -> 7 [label=conj_negcc]
	8 [label="."]
		2 -> 8 [label=punct]
}
'''.strip()

def test_conll_as_dotgraph_custom_digraph_and_idprefix():
    if older_than_py27: # this feature disabled in older Pythons
        return
    formatted_dotgraph = '''
digraph test {
	x0 [label=root]
	x1 [label=Cathy]
		x2 -> x1 [label=su]
	x2 [label=zag]
		x0 -> x2 [label=ROOT]
	x3 [label=hen]
		x2 -> x3 [label=obj1]
	x4 [label=wild]
		x5 -> x4 [label=mod]
	x5 [label=zwaaien]
		x2 -> x5 [label=vc]
	x6 [label="."]
		x5 -> x6 [label=punct]
}'''.strip()

    sentence = Sentence.from_conll(conll_example.splitlines())
    dotgraph = sentence.as_dotgraph(id_prefix='x',
                                    digraph_kwargs={'name': 'test'})

def test_conll_as_dotgraph_custom_nodeformat():
    if older_than_py27: # this feature disabled in older Pythons
        return
    formatted_dotgraph = '''
digraph {
	0 [label=root color=red]
	1 [label=Cathy color=green]
		2 -> 1 [label=su]
	2 [label=zag color=blue]
		0 -> 2 [label=ROOT]
	3 [label=hen]
		2 -> 3 [label=obj1]
	4 [label=wild]
		5 -> 4 [label=mod]
	5 [label=zwaaien color=green]
		2 -> 5 [label=vc]
	6 [label="."]
		5 -> 6 [label=punct]
}
'''.strip()

    def node_formatter(token):
        if token is None:
            return {'color': 'red'}
        elif token.cpos == 'N':
            return {'color': 'green'}
        elif token.cpos == 'V':
            return {'color': 'blue'}
        else:
            return {}
    sentence = Sentence.from_conll(conll_example.splitlines())
    dotgraph = sentence.as_dotgraph(node_formatter=node_formatter)
    assert dotgraph.source == formatted_dotgraph

def test_conll_as_dotgraph_custom_edgeformat():
    if older_than_py27: # this feature disabled in older Pythons
        return
    formatted_dotgraph = '''
digraph {
	0 [label=root]
	1 [label=Cathy]
		2 -> 1 [label=su color=blue]
	2 [label=zag]
		0 -> 2 [label=ROOT color=red]
	3 [label=hen]
		2 -> 3 [label=obj1 color=blue]
	4 [label=wild]
		5 -> 4 [label=mod color=blue]
	5 [label=zwaaien]
		2 -> 5 [label=vc color=blue]
	6 [label="."]
		5 -> 6 [label=punct color=blue]
}
'''.strip()

    sentence = Sentence.from_conll(conll_example.splitlines())
    def edge_formatter(token):
        if token.head == 0:
            return {'color': 'red'}
        else:
            return {'color': 'blue'}
    dotgraph = sentence.as_dotgraph(edge_formatter=edge_formatter)
    assert dotgraph.source == formatted_dotgraph

def test_read_sd_sentence():
    sample_deps = '''
det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)
    '''.strip().splitlines()
    sentence = Sentence.from_stanford_dependencies(sample_deps, tree4)
    assert stringify_sentence(sentence) == tree4_out_CCprocessed

def test_read_sd_sentence_extra_space():
    sample_deps = '''

det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)


    '''.splitlines()
    sentence = Sentence.from_stanford_dependencies(sample_deps, tree4)
    assert stringify_sentence(sentence) == tree4_out_CCprocessed

def test_read_sd_sentence_punct():
    sample_deps = '''
root(ROOT-0, Sentences-1)
punct(Sentences-1, :-2)
dep(Sentences-1, words-3)
punct(sometimes-8, -LRB--4)
prep(sometimes-8, with-5)
pobj(with-5, punctuation-6)
punct(sometimes-8, ---7)
dep(words-3, sometimes-8)
punct(sometimes-8, -RRB--9)
punct(Sentences-1, .-10)
    '''.strip().splitlines()
    tree = '''(ROOT (NP (NP (NNS Sentences)) (: :) (NP (NP (NNS words))
    (PRN (-LRB- -LRB-) (FRAG (PP (IN with) (NP (NN punctuation))) (: --)
    (ADVP (RB sometimes))) (-RRB- -RRB-))) (. .)))'''
    output = '''
Token(index=1, form='Sentences', cpos='NNS', pos='NNS', head=0, deprel='root')
Token(index=2, form=':', cpos=':', pos=':', head=1, deprel='punct')
Token(index=3, form='words', cpos='NNS', pos='NNS', head=1, deprel='dep')
Token(index=4, form='-LRB-', cpos='-LRB-', pos='-LRB-', head=8, deprel='punct')
Token(index=5, form='with', cpos='IN', pos='IN', head=8, deprel='prep')
Token(index=6, form='punctuation', cpos='NN', pos='NN', head=5, deprel='pobj')
Token(index=7, form='--', cpos=':', pos=':', head=8, deprel='punct')
Token(index=8, form='sometimes', cpos='RB', pos='RB', head=3, deprel='dep')
Token(index=9, form='-RRB-', cpos='-RRB-', pos='-RRB-', head=8, deprel='punct')
Token(index=10, form='.', cpos='.', pos='.', head=1, deprel='punct')
    '''.strip()

    sentence = Sentence.from_stanford_dependencies(sample_deps, tree)
    assert stringify_sentence(sentence) == output

    output_no_punct = '''
Token(index=1, form='Sentences', cpos='NNS', pos='NNS', head=0, deprel='root')
Token(index=3, form='words', cpos='NNS', pos='NNS', head=1, deprel='dep')
Token(index=5, form='with', cpos='IN', pos='IN', head=8, deprel='prep')
Token(index=6, form='punctuation', cpos='NN', pos='NN', head=5, deprel='pobj')
Token(index=8, form='sometimes', cpos='RB', pos='RB', head=3, deprel='dep')
    '''.strip()
    sentence2 = Sentence.from_stanford_dependencies(sample_deps, tree,
                                                    include_punct=False)
    assert stringify_sentence(sentence2) == output_no_punct

    sentence3 = Sentence.from_stanford_dependencies(sample_deps, tree,
                                                    include_punct=False,
                                                    include_erased=True)
    assert stringify_sentence(sentence3) == output_no_punct

    sentence4 = Sentence.from_stanford_dependencies(sample_deps, tree,
                                                    include_punct=True,
                                                    include_erased=True)
    assert stringify_sentence(sentence4) == output

    tree2 = '(ROOT(NP(NP-SBJ(NNS Sentences))(: :)(NP(NP(NNS words))(PRN(-LRB- -LRB-)(FRAG(PP(IN with)(NP(NN punctuation)))(: --)(ADVP(RB sometimes)))(-RRB- -RRB-)))(. .)))'
    sentence5 = Sentence.from_stanford_dependencies(sample_deps, tree2)
    assert sentence5 == sentence

    tree3 = '((NP(NP(NNS Sentences))(: :)(NP(NP(NNS words))(PRN(-LRB- -LRB-)(FRAG(PP(IN with)(NP(NN punctuation)))(: --)(ADVP(RB sometimes)))(-RRB- -RRB-)))(. .)))'
    sentence6 = Sentence.from_stanford_dependencies(sample_deps, tree3)
    assert sentence6 == sentence

    tree4 = ' ( ROOT(NP   ( NP-SBJ (NNS Sentences))(: :)(\nNP\n(NP(NNS\nwords)) (PRN (-LRB- -LRB-)(FRAG(PP    (IN with\n)\t(NP(NN punctuation )))(: --)(ADVP( RB sometimes )))(-RRB-    \t-RRB-)))(.\n\n\t.)))    '
    sentence7 = Sentence.from_stanford_dependencies(sample_deps, tree4)
    assert sentence7 == sentence

def test_read_sd_sentence_sorting():
    # same as test_read_sd_sentence but check sorting
    sample_deps = '''
punct(burrito-2, .-8)
conj_negcc(beans-4, chicken-7)
prep_with(burrito-2, chicken-7)
prep_with(burrito-2, beans-4)
root(ROOT-0, burrito-2)
det(burrito-2, A-1)
    '''.strip().splitlines()
    sentence = Sentence.from_stanford_dependencies(sample_deps, tree4)
    assert stringify_sentence(sentence) == tree4_out_CCprocessed

def test_read_sd_corpus_single():
    sample_deps = '''
det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)
    '''.strip().splitlines()
    corpus = Corpus.from_stanford_dependencies(sample_deps, [tree4])
    assert len(corpus) == 1
    assert stringify_sentence(corpus[0]) == tree4_out_CCprocessed

def test_read_sd_corpus_multiple():
    sample_deps = '''
det(burrito-2, A-1)
root(ROOT-0, burrito-2)
prep_with(burrito-2, beans-4)
prep_with(burrito-2, chicken-7)
conj_negcc(beans-4, chicken-7)
punct(burrito-2, .-8)

nsubj(cooks-2, Ed-1)                          
nsubj(sells-4, Ed-1)
root(ROOT-0, cooks-2)
conj_and(cooks-2, sells-4)
dobj(cooks-2, burritos-5)
prep_with(burritos-5, beans-7)
prep_with(burritos-5, rice-10)
conj_negcc(beans-7, rice-10)
punct(cooks-2, .-11)
    '''.strip().splitlines()
    corpus = Corpus.from_stanford_dependencies(sample_deps, [tree4, tree5])
    assert len(corpus) == 2
    assert stringify_sentence(corpus[0]) == tree4_out_CCprocessed
    assert stringify_sentence(corpus[1]) == tree5_out_CCprocessed



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jpype
from .StanfordDependencies import (StanfordDependencies,
                                   JavaRuntimeVersionError)
from .CoNLL import Token, Sentence

class JPypeBackend(StanfordDependencies):
    """Faster backend than SubprocessBackend but requires you to install
    jpype ('pip install JPype1', not 'JPype'). May be less stable. There's
    no speed benefit of using convert_trees() over convert_tree() for this
    backend. In terms of output, should be identical to SubprocessBackend
    except that all string fields will be unicode. Additionally, has
    the option to run the lemmatizer (see convert_tree())."""
    def __init__(self, jar_filename=None, download_if_missing=False,
                 version=None, extra_jvm_args=None, start_jpype=True,
                 jvm_path=None):
        """extra_jvm_args can be set to a list of strings which will
        be passed to your JVM.  If start_jpype is True, we will start
        a JVM via JPype if one hasn't been started already. The user is
        responsible for stopping the JVM (jpype.shutdownJVM()) when they
        are done converting. Once the JVM has been shutdown, you'll need
        to create a new JPypeBackend in order to convert after that.
        jvm_path is the path to libjvm.so (if None, will use JPype's
        default JRE path)."""
        StanfordDependencies.__init__(self, jar_filename, download_if_missing,
                                      version)
        if start_jpype and not jpype.isJVMStarted():
            jpype.startJVM(jvm_path or jpype.getDefaultJVMPath(),
                           '-ea',
                           '-Djava.class.path=' + self.jar_filename,
                           *(extra_jvm_args or []))
        self.corenlp = jpype.JPackage('edu').stanford.nlp
        try:
            self.acceptFilter = self.corenlp.util.Filters.acceptFilter()
        except TypeError:
            # this appears to be caused by a mismatch between CoreNLP and JRE
            # versions since this method changed to return a Predicate.
            version = jpype.java.lang.System.getProperty("java.version")
            self._report_version_error(version)
        trees = self.corenlp.trees
        self.treeReader = trees.Trees.readTree
        self.grammaticalStructure = trees.EnglishGrammaticalStructure
        self.stemmer = self.corenlp.process.Morphology.stemStaticSynchronized
        puncFilterInstance = trees.PennTreebankLanguagePack(). \
            punctuationWordRejectFilter()
        try:
            self.puncFilter = puncFilterInstance.test
        except AttributeError:
            self.puncFilter = puncFilterInstance.accept
        self.lemma_cache = {}
    def convert_tree(self, ptb_tree, representation='basic',
                     include_punct=True, include_erased=False,
                     add_lemmas=False):
        """Arguments are as in StanfordDependencies.convert_trees but with
        the addition of add_lemmas. If add_lemmas=True, we will run the
        Stanford CoreNLP lemmatizer and fill in the lemma field."""
        self._raise_on_bad_representation(representation)
        tree = self.treeReader(ptb_tree)
        deps = self._get_deps(tree, include_punct, representation)

        indices_to_words = dict(enumerate(tree.taggedYield(), 1))
        tokens = Sentence()
        covered_indices = set()

        def add_token(index, form, head, deprel):
            tag = indices_to_words[index].tag()
            if add_lemmas:
                lemma = self.stem(form, tag)
            else:
                lemma = None
            token = Token(index=index, form=form, lemma=lemma, cpos=tag,
                          pos=tag, feats=None, head=head, deprel=deprel,
                          phead=None, pdeprel=None)
            tokens.append(token)

        # add token for each dependency
        for dep in deps:
            index = dep.dep().index()
            head = dep.gov().index()
            deprel = dep.reln().toString()
            form = indices_to_words[index].value()

            add_token(index, form, head, deprel)
            covered_indices.add(index)
        if include_erased:
            # see if there are any tokens that were erased
            # and add them as well
            all_indices = set(indices_to_words.keys())
            for index in all_indices - covered_indices:
                form = indices_to_words[index].value()
                if not include_punct and not self.puncFilter(form):
                    continue
                add_token(index, form, head=0, deprel='erased')
            # erased generally disrupt the ordering of the sentence
            tokens.sort()
        return tokens
    def stem(self, form, tag):
        """Returns the stem of word with specific form and part-of-speech
        tag according to the Stanford lemmatizer. Lemmas are cached."""
        key = (form, tag)
        if key not in self.lemma_cache:
            lemma = self.stemmer(*key).word()
            self.lemma_cache[key] = lemma
        return self.lemma_cache[key]

    def _get_deps(self, tree, include_punct, representation):
        """Get a list of dependencies from a Stanford Treee for a specific
        Stanford Dependencies representation."""
        if include_punct:
            egs = self.grammaticalStructure(tree, self.acceptFilter)
        else:
            egs = self.grammaticalStructure(tree)

        if representation == 'basic':
            deps = egs.typedDependencies()
        elif representation == 'collapsed':
            deps = egs.typedDependenciesCollapsed(True)
        elif representation == 'CCprocessed':
            deps = egs.typedDependenciesCCprocessed(True)
        else:
            # _raise_on_bad_representation should ensure that this
            # assertion doesn't fail
            assert representation == 'collapsedTree'
            deps = egs.typedDependenciesCollapsedTree()
        return deps

    @staticmethod
    def _report_version_error(version):
        print "Your Java version:", version
        if version.split('.')[:2] < ['1', '8']:
            print "The last CoreNLP release for Java 1.6/1.7 was 3.4.1"
            print "Try using: StanfordDependencies.get_instance(" \
                  "backend='jpype', version='3.4.1')"
            print
        raise JavaRuntimeVersionError()



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
PyStanfordDependencies
======================

Python interface for converting `Penn Treebank
<http://www.cis.upenn.edu/~treebank/>`_ trees to `Stanford Dependencies
<http://nlp.stanford.edu/software/stanford-dependencies.shtml>`_.

Example usage
-------------
Start by getting a ``StanfordDependencies`` instance with
``StanfordDependencies.get_instance()``::

    >>> import StanfordDependencies
    >>> sd = StanfordDependencies.get_instance(backend='subprocess')

``get_instance()`` takes several options. ``backend`` can currently
be ``subprocess`` or ``jpype`` (see below). If you have an existing
`Stanford CoreNLP <http://nlp.stanford.edu/software/corenlp.shtml>`_ or
`Stanford Parser <http://nlp.stanford.edu/software/lex-parser.shtml>`_
jar file, use the ``jar_filename`` parameter to point to the full path of
the jar file. Otherwise, PyStanfordDependencies will download a jar file
for you and store it in locally (``~/.local/share/pystanforddeps``). You
can request a specific version with the ``version`` flag, e.g.,
``version='3.4.1'``. To convert trees, use the ``convert_trees()`` or
``convert_tree()`` method (note that by default, ``convert_trees()`` can
be considerably faster if you're doing batch conversion). These return
a sentence (list of ``Token`` objects) or a list of sentences (list of
list of ``Token`` objects) respectively::

    >>> sent = sd.convert_tree('(S1 (NP (DT some) (JJ blue) (NN moose)))')
    >>> for token in sent:
    ...     print token
    ...
    Token(index=1, form='some', cpos='DT', pos='DT', head=3, deprel='det')
    Token(index=2, form='blue', cpos='JJ', pos='JJ', head=3, deprel='amod')
    Token(index=3, form='moose', cpos='NN', pos='NN', head=0, deprel='root')

This tells you that ``moose`` is the head of the sentence and is
modified by ``some`` (with a ``det`` = determiner relation) and ``blue``
(with an ``amod`` = adjective modifier relation). Fields on ``Token``
objects are readable as attributes. See docs for additional options in
``convert_tree()`` and ``convert_trees()``.

Visualization
-------------

If you have the `asciitree <https://pypi.python.org/pypi/asciitree>`_
package, you can use a prettier ASCII formatter::

    >>> print sent.as_asciitree()
     moose [root]
      +-- some [det]
      +-- blue [amod]

If you have Python 2.7 or later, you can use `Graphviz
<http://graphviz.org/>`_ to render your graphs. You'll need the `Python
graphviz <https://pypi.python.org/pypi/graphviz>`_ package to call
``as_dotgraph()``::

    >>> dotgraph = sent.as_dotgraph()
    >>> print dotgraph
    digraph {
            0 [label=root]
            1 [label=some]
                    3 -> 1 [label=det]
            2 [label=blue]
                    3 -> 2 [label=amod]
            3 [label=moose]
                    0 -> 3 [label=root]
    }
    >>> dotgraph.render('moose') # renders a PDF by default
    'moose.pdf'
    >>> dotgraph.format = 'svg'
    >>> dotgraph.render('moose')
    'moose.svg'

The Python `xdot <https://pypi.python.org/pypi/xdot>`_
package provides an interactive visualization::

    >>> import xdot
    >>> window = xdot.DotWindow()
    >>> window.set_dotcode(dotgraph.source)

Both ``as_asciitree()`` and ``as_dotgraph()`` allow customization.
See the docs for additional options.

Backends
--------
Currently PyStanfordDependencies includes two backends:

- ``subprocess`` (works anywhere with a ``java`` binary, slow so
  batched conversions with ``convert_trees()`` are recommended)
- ``jpype`` (requires `jpype1 <https://pypi.python.org/pypi/JPype1>`_,
  faster than Subprocess, includes access to the Stanford CoreNLP
  lemmatizer)

By default, PyStanfordDependencies will attempt to use the ``jpype``
backend. If ``jpype`` isn't available or crashes on startup,
PyStanfordDependencies will fallback to ``subprocess`` with a warning.

More information
----------------
Licensed under `Apache 2.0 <http://www.apache.org/licenses/LICENSE-2.0>`_.

Written by David McClosky (`homepage
<http://nlp.stanford.edu/~mcclosky/>`_, `code <http://github.com/dmcc>`_)

Bug reports and feature requests: `GitHub issue tracker
<http://github.com/dmcc/PyStanfordDependencies/issues>`_

Release summaries
-----------------
- 0.1.6 (2015.02.12): Support for graphviz formatting, CoreNLP 3.5.1,
  better Windows portability
- 0.1.5 (2015.01.10): Support for ASCII tree formatting
- 0.1.4 (2015.01.07): Fix CCprocessed support
- 0.1.3 (2015.01.03): Bugfixes, coveralls integration, refactoring
- 0.1.2 (2015.01.02): Better CoNLL structures, test suite and Travis-CI
  support, bugfixes
- 0.1.1 (2014.12.15): More docs, fewer bugs
- 0.1 (2014.12.14): Initial version
"""

from .StanfordDependencies import (StanfordDependencies, get_instance,
                                   JavaRuntimeVersionError)
from .CoNLL import Corpus, Sentence, Token
__all__ = (StanfordDependencies, get_instance, JavaRuntimeVersionError,
           Corpus, Sentence, Token)

__authors__ = 'David McClosky'
__license__ = 'Apache 2.0'
__version__ = '0.1.6'
__email__ = 'notsoweird+pystanforddependencies@gmail.com'
